from typing import Final


__author__: Final[str] = "Tsuyoshi Hombashi"
__copyright__: Final[str] = f"Copyright 2023, {__author__}"
__license__: Final[str] = "MIT License"
__version__: Final[str] = "0.0.1"
__maintainer__: Final[str] = __author__
__email__: Final[str] = "tsuyoshi.hombashi@gmail.com"
